#!/usr/bin/python3 -t
# -*- coding: utf-8 -*-

import boto3
import logging
import subprocess
import json
import uuid
import argparse
import os
import ipaddress

PSKD='DEGUPSK'
JOIN_TIMEOUT='30'
CONFIG_DIR="/etc/coap-mqtt/"

class DeguMgr:
    def __init__(self):
        f = open(CONFIG_DIR + 'mqttinfo.json')
        p = json.load(f)
        self.iot = boto3.client('iot',
                                aws_access_key_id=p['accesskeyid'],
                                aws_secret_access_key=p['secretaccesskey'],
                                region_name=p['region']
        )
        self.accesskeyid = p['accesskeyid']
        self.region = p['region']

    def exists(self, name):
        list_of_things = self.iot.list_things()
        for thing in list_of_things['things']:
            if thing['thingName'] == name:
                return True
        while 'nextToken' in list_of_things:
            t = list_of_things['nextToken']
            list_of_things = self.iot.list_things(nextToken=t)
            for thing in list_of_things['things']:
                if thing['thingName'] == name:
                    return True
        return False

    def prepare_thing_types(self):
        self.iot.create_thing_type(thingTypeName='Degu')
        self.iot.create_thing_type(thingTypeName='Armadillo')

    def register_gw(self, mac):
        self.prepare_thing_types()
        name = 'Armadillo-'+mac
        if self.exists(name):
            print("gateway already exists")
            return False
        resp =  self.iot.create_thing(thingName=name, thingTypeName='Armadillo')
        arn = resp['thingArn']
        result = self.create_keys_and_certificate()

        cert_id = result['certificateId']
        cert_id_filename = CONFIG_DIR +  'gw_cert_id_file'
        cert_id_file = open(cert_id_filename, 'w')
        cert_id_file.write(cert_id)
        cert_id_file_path = os.path.abspath(cert_id_filename)
        os.chmod(cert_id_file_path, 0o600)
        cert_id_file.close()

        cert_pem = result['certificatePem']
        cert_pem_filename = CONFIG_DIR + 'certificate.pem.crt'
        cert_pem_file = open(cert_pem_filename, 'w')
        cert_pem_file.write(cert_pem)
        cert_pem_file_path = os.path.abspath(cert_pem_filename)
        os.chmod(cert_pem_file_path, 0o600)
        cert_pem_file.close()

        private_key_pem = result['keyPair']['PrivateKey']
        private_key_pem_filename = CONFIG_DIR + 'private.pem.key'
        private_key_pem_file = open(private_key_pem_filename, 'w')
        private_key_pem_file.write(private_key_pem)
        private_key_pem_file_path = os.path.abspath(private_key_pem_filename)
        os.chmod(private_key_pem_file_path, 0o600)
        private_key_pem_file.close()

        private_key_pem_filename = '/usr/share/degu-manager/policy_document.templ'
        private_key_pem_file = open(private_key_pem_filename, 'r')
        policy_document_text = private_key_pem_file.read()
        try:
            self.iot.create_policy(policyName='gw', policyDocument=policy_document_text)
        except:
            pass
        result = self.iot.describe_certificate(certificateId=cert_id)
        arn = result['certificateDescription']['certificateArn']
        self.arn = arn
        result = self.iot.attach_principal_policy(policyName='gw', principal=arn)
        return True

    def register_degu(self, mac):
        self.prepare_thing_types()
        try:
            resp = self.iot.create_thing(thingName=mac,thingTypeName='Degu')
        except Exception as e:
            if e.response['Error']['Code'] == 'ResourceAlreadyExistsException':
                print("User already exists")
            else:
                print("Unexpected error: %s" % e)
        cert_id_filename = CONFIG_DIR +  'gw_cert_id_file'
        cert_id_file = open(cert_id_filename, 'r')
        cert_id = cert_id_file.read()
        cert_id_file.close()
        result = self.iot.describe_certificate(certificateId=cert_id)
        arn = result['certificateDescription']['certificateArn']
        self.iot.attach_thing_principal(thingName=mac, principal=arn)
        return True

    def unregister_gw(self, mac):
        try:
            resp = self.iot.delete_thing(thingName='Armadillo-'+mac)
        except Exception as e:
            if e.response['Error']['Code'] == 'ResourceAlreadyExistsException':
                print("User already exists")
            else:
                print("Unexpected error: %s" % e)

    def unregister_degu(self, mac):
        try:
            resp = self.iot.delete_thing(thingName=mac)
        except Exception as e:
            if e.response['Error']['Code'] == 'ResourceAlreadyExistsException':
                print("User already exists")
            else:
                print("Unexpected error: %s" % e)

    def list_degu(self):
        things = self.iot.list_things(thingTypeName='Degu',  maxResults=2)
        for item in things['things']:
            print(item['thingName'])
        while 'nextToken' in things:
            t = things['nextToken']
            things = self.iot.list_things(thingTypeName='Degu',  maxResults=2, nextToken=t)
            for item in things['things']:
                print(item['thingName'])

    def commission_enable(self):
        proc = subprocess.Popen(['wpanctl','commissioner', 'start'],
                                stdout=subprocess.PIPE)
        while True:
            line = proc.stdout.readline()
            if line != b'':
                print(line)
            else:
                break

    def commission_disable(self):
        proc = subprocess.Popen(['wpanctl','commissioner', 'stop'],
                                stdout=subprocess.PIPE)
        while True:
            line = proc.stdout.readline()
            if line != b'':
                print(line)
            else:
                break

    def commission_joiner(self, eui, pskd):
        proc = subprocess.Popen(['wpanctl','commissioner', 'joiner-add',
                                 eui, '2000', pskd],stdout=subprocess.PIPE)
        while True:
            line = proc.stdout.readline()
            if line != b'':
                print(line)
            else:
                break

    def create_keys_and_certificate(self):
        asset = self.iot.create_keys_and_certificate(setAsActive=True)
        self.id = asset["certificateId"]
        self.arn = asset["certificateArn"]
        return asset

    def attach_gw(self, mac):
        self.iot.attach_thing_principal(thingName="Armadillo-" + mac,
                                        principal=self.arn)

def gen_static_gw_address():
    out = subprocess.getoutput("wpanctl form -p 0xABCD degu")

    out = subprocess.getoutput(
        """ip -6 addr show dev wpan0 scope global |
        grep inet6 | head -1 |awk '{print $2}'""")

    one  = ipaddress.IPv6Address('::1')
    a = ipaddress.ip_interface(out)
    gw_address = str(ipaddress.IPv6Address(int(a.ip) &
                                           int(a.netmask) | int(one))) + '/64'
    out = subprocess.getoutput("ip -6 addr add "
                               + gw_address +
                               " dev wpan0")



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--register-degu", metavar="MAC", help="register degu by MAC")
    parser.add_argument("--register-gw", action='store_true')
    args = parser.parse_args()

    if args.register_degu:
        mgr = DeguMgr()
        mgr.register_degu(args.register_degu)
        mgr.commission_disable()
        mgr.commission_enable()
        mgr.commission_joiner(args.register_degu, 'DEGUPSK')

    if args.register_gw:
        gen_static_gw_address()
        mgr = DeguMgr()
        gw_mac = subprocess.getoutput(
            """ip link show dev eth0 | grep link | awk '{print toupper($2)}' | tr -d ':'""")
        if mgr.register_gw(gw_mac):
            mgr.attach_gw(gw_mac)
    exit(0)
